'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

exports.default = function (classString, options) {
  var parts = (classString || '').split(/(\S+\s+)/).filter(function (s) {
    return s;
  });
  var node = void 0;
  var column = 0;
  var nodes = [];
  parts.forEach(function (part) {
    var trailingSpace = part.match(/\s+$/);
    trailingSpace = trailingSpace && trailingSpace[0];
    if (isLogicalIf(part)) {
      var _part$trim$split = part.trim().split(':'),
          _part$trim$split2 = (0, _toArray3.default)(_part$trim$split),
          pathStr = _part$trim$split2[0],
          values = _part$trim$split2.slice(1);

      var isUnless = values[0] === '';
      if (isUnless) {
        values.splice(0, 1); // remove leading blank that is not used in unless
      }
      var helperName = isUnless ? 'unless' : 'if';
      var startMust = column;
      column += 2; // "{{"
      var startIf = column;
      column += helperName.length;
      var endIf = column;
      // " " between {{if thing "then" "else"}}
      //                 ^
      column += 1; // " "
      var startPath = column;
      column += pathStr.length;
      var endPath = column;
      var path = _glimmerSyntax.builders.path(pathStr, _glimmerSyntax.builders.loc(1, startPath, 1, endPath));

      var params = values.map(function (value, i) {
        var str = _glimmerSyntax.builders.string(value);
        var length = value.length;
        if (i < values.length) {
          // leading " " before {{if thing "then" "else"}}
          //                              ^      ^
          column += 1;
        }
        var startStr = column;
        column += length + 2; // plus 2 quotes
        var endStr = column;

        str.loc = _glimmerSyntax.builders.loc(1, startStr, 1, endStr);

        return str;
      });

      column += 2; // "}}"
      var endMust = column;

      node = _glimmerSyntax.builders.mustache(helperName, [path].concat((0, _toConsumableArray3.default)(params)), null, null, _glimmerSyntax.builders.loc(1, startMust, 1, endMust));

      // "if" or "unless" location
      node.path.loc = _glimmerSyntax.builders.loc(1, startIf, 1, endIf);

      nodes.push(node);

      if (trailingSpace) {
        column = pushText(trailingSpace, column, nodes, options);
      }
    } else if (isStatic(part)) {
      var value = part.replace(/^:/, '');
      column = pushText(value, column, nodes, options);
    } else {
      column = pushBound(part.trim(), column, nodes, options);
      if (trailingSpace) {
        column = pushText(trailingSpace, column, nodes, options);
      }
    }
  });
  return nodes;
};

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isLogicalIf(str) {
  return (/(.+:.+:.+|.+:.+)/.test(str)
  );
}

function isStatic(str) {
  return (/:.+/.test(str)
  );
}

function pushBound(value, column, nodes) {
  var columnEnd = column + value.length + 4; // {{}} = 4
  var node = _glimmerSyntax.builders.mustache(value);
  node.loc = _glimmerSyntax.builders.loc(1, column, 1, columnEnd);
  node.path.loc = _glimmerSyntax.builders.loc(1, column + 2, 1, columnEnd - 2);
  nodes.push(node);
  return columnEnd;
}

function pushText(value, column, nodes) {
  var columnEnd = column + value.length;
  var node = _glimmerSyntax.builders.text(value);
  node.loc = _glimmerSyntax.builders.loc(1, column, 1, columnEnd);
  nodes.push(node);
  return columnEnd;
}