'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.default = print;

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _underscore = require('underscore');

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

var _isSelfClosing = require('./utils/is-self-closing');

var _isSelfClosing2 = _interopRequireDefault(_isSelfClosing);

var _escapeHtml = require('./utils/escape-html');

var _escapeHtml2 = _interopRequireDefault(_escapeHtml);

var _whitespaceDiff = require('./utils/whitespace-diff');

var _whitespaceDiff2 = _interopRequireDefault(_whitespaceDiff);

var _node = require('./utils/node');

var _location = require('./utils/location');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-use-before-define: ["error", { "functions": false }] */
/* eslint no-param-reassign: [0, { "ignorePropertyModificationsFor": ["lastLoc"] }] */
var defaults = { quotes: { mustache: '"' } };

function print(ast) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var optionsWithDefault = (0, _deepmerge2.default)(defaults, options);
  var lastLoc = _glimmerSyntax.builders.loc(1, 0, 1, 0);
  return build(ast, lastLoc, optionsWithDefault).output.join('');
}

function buildEach(asts, lastLoc, options) {
  var output = [];
  asts.forEach(function (node) {
    var _build = build(node, lastLoc, options),
        out = _build.output,
        loc = _build.lastLoc;

    output.push.apply(output, (0, _toConsumableArray3.default)(out));
    lastLoc = loc;
  });
  return { lastLoc: lastLoc, output: output };
}

function pathParams(ast, lastLoc, options) {
  var output = [];
  if (ast.name) {
    var _build2 = build(ast.name, lastLoc, options),
        name = _build2.output,
        nameLoc = _build2.lastLoc;

    output.push.apply(output, (0, _toConsumableArray3.default)(name));
    lastLoc = nameLoc || lastLoc;
  }
  if (ast.path) {
    var _build3 = build(ast.path, lastLoc, options),
        path = _build3.output,
        pathLoc = _build3.lastLoc;

    output.push.apply(output, (0, _toConsumableArray3.default)(path));
    lastLoc = pathLoc || lastLoc;
  }
  if (ast.params) {
    var _buildEach = buildEach(ast.params, lastLoc, options),
        params = _buildEach.output,
        paramsLoc = _buildEach.lastLoc;

    output.push.apply(output, (0, _toConsumableArray3.default)(params));
    lastLoc = paramsLoc || lastLoc;
  }
  if (ast.hash) {
    var _build4 = build(ast.hash, lastLoc, options),
        hash = _build4.output,
        hashLoc = _build4.lastLoc;

    output.push.apply(output, (0, _toConsumableArray3.default)(hash));
    lastLoc = hashLoc || lastLoc;
  }
  return { output: output, lastLoc: lastLoc };
}

function blockParams(block, lastLoc) {
  var params = block.program.blockParams;
  if (params.length) {
    var spaced = (0, _underscore.reduce)(params, function (acc, p) {
      return acc.concat([p, ' ']);
    }, []);
    spaced.splice(-1, 1);
    var length = (0, _underscore.reduce)(spaced, function (acc, p) {
      return acc + p.length;
    }, 0);
    lastLoc = (0, _location.locAppend)(lastLoc.end, { column: length + 5 });
    return { lastLoc: lastLoc, output: [' as |'].concat((0, _toConsumableArray3.default)(spaced), ['|']) };
  }
  return { lastLoc: lastLoc, output: [] };
}

function openBlock(block, lastLoc, options) {
  lastLoc = (0, _location.locAppend)(lastLoc.start, { column: 3 }); // {{#

  var _pathParams = pathParams(block, lastLoc, options),
      path = _pathParams.output,
      pathLoc = _pathParams.lastLoc;

  lastLoc = pathLoc;

  var _blockParams = blockParams(block, lastLoc, options),
      params = _blockParams.output,
      paramsLoc = _blockParams.lastLoc;

  lastLoc = paramsLoc;
  lastLoc = (0, _location.locAppend)(lastLoc.end, { column: 2 }); // }}
  return { lastLoc: lastLoc, output: ['{{#'].concat((0, _toConsumableArray3.default)(path), (0, _toConsumableArray3.default)(params), ['}}']) };
}

function closeBlock(block, lastLoc, options) {
  lastLoc = (0, _location.locAppend)(lastLoc.start, { column: 3 }); // {{/

  var _build5 = build(block.path, lastLoc, options),
      path = _build5.output;

  return ['{{/'].concat((0, _toConsumableArray3.default)(path), ['}}']);
}

function build(ast, lastLoc, options) {
  if (!ast) {
    return '';
  }
  var output = [];

  if (ast.loc && !ast.skipInitialWhitespace) {
    output.push((0, _whitespaceDiff2.default)(lastLoc, ast.loc));
  }
  if (ast.skipInitialWhitespace) {
    delete ast.skipInitialWhitespace;
  }
  lastLoc = ast.loc || lastLoc;

  switch (ast.type) {
    case 'Program':
      {
        var chainBlock = ast.chained && ast.body[0];
        if (chainBlock) {
          chainBlock.chained = true;
        }

        var _buildEach2 = buildEach(ast.body, lastLoc, options),
            body = _buildEach2.output;

        output.push.apply(output, (0, _toConsumableArray3.default)(body));
      }
      break;
    case 'ElementNode':
      {
        var selfClosing = (0, _isSelfClosing2.default)(ast.tag);
        var attrNodes = (0, _node.sortNodes)(ast.attributes, ast.modifiers, ast.comments);

        output.push('<', ast.tag);

        var loc = lastLoc;
        var tagLength = ast.tag.length;
        var tagLoc = (0, _location.locAppend)(loc.start, { column: tagLength + 1 });
        loc = tagLoc;

        if (attrNodes.length) {
          var _buildEach3 = buildEach(attrNodes, loc, options),
              attrs = _buildEach3.output,
              attrsLoc = _buildEach3.lastLoc;

          output.push.apply(output, (0, _toConsumableArray3.default)(attrs));
          loc = attrsLoc;
        }

        output.push('>');
        loc = (0, _location.locAppend)(loc.end, { column: 1 }); // >
        if (!selfClosing) {
          var _buildEach4 = buildEach(ast.children, loc, options),
              children = _buildEach4.output,
              childLoc = _buildEach4.lastLoc;

          output.push.apply(output, (0, _toConsumableArray3.default)(children));
          output.push('</', ast.tag, '>');
          loc = (0, _location.locAppend)(childLoc.end, { column: 3 + ast.tag.length }); // </tag>
          lastLoc = loc;
        }
      }
      break;
    case 'AttrNode':
      {
        output.push(ast.name, '=');
        var _loc = (0, _location.locAppend)(lastLoc.start, { column: ast.name.length + 1 });

        var _build6 = build(ast.value, _loc, options),
            value = _build6.output;

        if (ast.value.type === 'TextNode') {
          output.push.apply(output, ['"'].concat((0, _toConsumableArray3.default)(value), ['"']));
        } else {
          output.push.apply(output, (0, _toConsumableArray3.default)(value));
        }
      }
      break;
    case 'ConcatStatement':
      {
        output.push('"');

        var _buildEach5 = buildEach(ast.parts, lastLoc, options),
            parts = _buildEach5.output,
            partsLoc = _buildEach5.lastLoc;

        output.push.apply(output, (0, _toConsumableArray3.default)(parts));
        output.push('"');
        lastLoc = _glimmerSyntax.builders.loc(partsLoc);
        lastLoc.end.column += 1; // "
      }
      break;
    case 'TextNode':
      output.push((0, _escapeHtml2.default)(ast.chars));
      break;
    case 'MustacheStatement':
      {
        var open = ast.escaped ? '{{' : '{{{';
        var close = ast.escaped ? '}}' : '}}}';
        var _loc2 = (0, _location.locAppend)(lastLoc.start, { column: open.length }); // {{ or {{{

        var _pathParams2 = pathParams(ast, _loc2, options),
            out = _pathParams2.output,
            pathLoc = _pathParams2.lastLoc;

        _loc2 = (0, _location.locAppend)(pathLoc.end, { column: close.length + 1 }); // }} or }}}

        var whitespace = '';
        if (ast.loc && _loc2) {
          var lineDiff = ast.loc.end.line - _loc2.end.line;
          var columnDiff = 0;
          if (lineDiff === 0) {
            columnDiff = ast.loc.end.column - _loc2.end.column;
          } else if (lineDiff > 0) {
            columnDiff = ast.loc.end.column - 2;
          }
          var offset = {
            start: {
              line: 0,
              column: 0
            },
            end: {
              line: lineDiff,
              column: columnDiff
            }
          };
          whitespace = (0, _whitespaceDiff.locToWhitespace)(offset);
          if (whitespace) {
            _loc2 = ast.loc;
          }
        }
        output.push.apply(output, [open].concat((0, _toConsumableArray3.default)(out), [whitespace, close]));
        lastLoc = _loc2;
      }
      break;
    case 'MustacheCommentStatement':
      {
        var contentLength = ast.value.length;
        var _ast$loc = ast.loc,
            start = _ast$loc.start.column,
            end = _ast$loc.end.column;

        var nodeLength = end - start;
        var diff = nodeLength - contentLength;
        var _open = diff > 5 ? '{{!--' : '{{!';
        var _close = diff > 5 ? '--}}' : '}}';
        output.push(_open, ast.value, _close);
      }
      break;
    case 'ElementModifierStatement':
      output.push.apply(output, ['{{'].concat((0, _toConsumableArray3.default)(pathParams(ast, lastLoc, options).output), ['}}']));
      break;
    case 'PathExpression':
      output.push(ast.original);
      break;
    case 'SubExpression':
      output.push.apply(output, ['('].concat((0, _toConsumableArray3.default)(pathParams(ast, lastLoc, options).output), [')']));
      break;
    case 'BooleanLiteral':
      output.push(ast.value ? 'true' : false);
      break;
    case 'BlockStatement':
      {
        var _openBlock = openBlock(ast, lastLoc, options),
            _open2 = _openBlock.output;

        output.push.apply(output, (0, _toConsumableArray3.default)(_open2));
        var firstBodyNode = ast.program.body[0];
        if (firstBodyNode) {
          firstBodyNode.skipInitialWhitespace = true;
        }

        var _build7 = build(ast.program, lastLoc, options),
            program = _build7.output;

        output.push.apply(output, (0, _toConsumableArray3.default)(program));

        if (ast.inverse) {
          if (!ast.inverse.chained) {
            output.push('{{else}}');
          }
          var firstInverseNode = ast.inverse.body[0];
          ast.inverse.skipInitialWhitespace = true;
          if (firstInverseNode) {
            firstInverseNode.skipInitialWhitespace = true;
          }

          var _build8 = build(ast.inverse, ast.inverse.loc, options),
              inverse = _build8.output;

          output.push.apply(output, (0, _toConsumableArray3.default)(inverse));
        }

        if (!ast.chained) {
          output.push.apply(output, (0, _toConsumableArray3.default)(closeBlock(ast, lastLoc, options)));
        }
      }
      break;
    case 'PartialStatement':
      {
        var _loc3 = (0, _location.locAppend)(lastLoc.start, { column: 3 }); // {{>

        var _pathParams3 = pathParams(ast, _loc3, options),
            params = _pathParams3.output;

        output.push.apply(output, ['{{>'].concat((0, _toConsumableArray3.default)(params), ['}}']));
      }
      break;
    case 'CommentStatement':
      {
        output.push('<!--', ast.value, '-->');
        var _lastLoc$end = lastLoc.end,
            line = _lastLoc$end.line,
            column = _lastLoc$end.column;
        // TODO: multiline comments

        lastLoc = _glimmerSyntax.builders.loc(line, column, line, column + ast.value.length + 7);
      }
      break;
    case 'StringLiteral':
      {
        var quote = options.quotes.mustache;
        output.push(quote, ast.value, quote);
      }
      break;
    case 'NumberLiteral':
      output.push(ast.value);
      break;
    case 'UndefinedLiteral':
      output.push('undefined');
      break;
    case 'NullLiteral':
      output.push('null');
      break;
    case 'Hash':
      {
        var _buildEach6 = buildEach(ast.pairs, lastLoc, options),
            pairs = _buildEach6.output,
            _loc4 = _buildEach6.lastLoc;

        output.push.apply(output, (0, _toConsumableArray3.default)(pairs));
        lastLoc = _loc4;
      }
      break;
    case 'HashPair':
      {
        var _build9 = build(ast.value, lastLoc, options),
            _value = _build9.output,
            _loc5 = _build9.lastLoc;

        output.push.apply(output, [ast.key, '='].concat((0, _toConsumableArray3.default)(_value)));
        lastLoc = _loc5;
      }
    // no default
  }
  return { lastLoc: lastLoc, output: output };
}