'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sizeOfNodes = exports.enumerate = exports.removeNode = exports.offsetNode = exports.nodeIndex = exports.nodesAfter = exports.nodesBefore = exports.sortNodes = undefined;

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _glimmerSyntax = require('glimmer-engine/dist/node_modules/glimmer-syntax');

var _location = require('./location');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable max-len */
/* eslint no-param-reassign: ["error", { "props": true, "ignorePropertyModificationsFor": ["parent", "offset", "node"] }] */
/* eslint-enable max-len */
function sortNodes() {
  for (var _len = arguments.length, nodes = Array(_len), _key = 0; _key < _len; _key++) {
    nodes[_key] = arguments[_key];
  }

  var allNodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  return _underscore2.default.uniq(allNodes).sort(function (a, b) {
    var lineDiff = a.loc.start.line - b.loc.start.line;
    var colDiff = a.loc.start.column - b.loc.start.column;
    return lineDiff === 0 && colDiff > 0 || lineDiff > 0;
  });
}

function nodeIndex(node) {
  for (var _len2 = arguments.length, lists = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    lists[_key2 - 1] = arguments[_key2];
  }

  var nodes = sortNodes.apply(undefined, [node].concat(lists));
  return nodes.indexOf(node);
}

function sameLine(node) {
  for (var _len3 = arguments.length, nodes = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    nodes[_key3 - 1] = arguments[_key3];
  }

  var allNodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  return sortNodes([node], allNodes);
}

function nodesBefore(before) {
  for (var _len4 = arguments.length, nodes = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    nodes[_key4 - 1] = arguments[_key4];
  }

  var allNodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  var sorted = sameLine(before, allNodes);
  var index = sorted.indexOf(before);
  var start = index;
  var currentLine = before.loc.start.line;
  for (var i = start; i >= 0; i -= 1) {
    var endLine = sorted[i].loc.end.line;
    if (endLine === currentLine) {
      start = i;
    } else {
      break;
    }
  }
  return sorted.slice(start, index);
}

function nodesAfter(after) {
  for (var _len5 = arguments.length, nodes = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
    nodes[_key5 - 1] = arguments[_key5];
  }

  var allNodes = nodes.reduce(function (acc, n) {
    return acc.concat(n);
  }, []);
  var sorted = sameLine(after, allNodes);
  var index = sorted.indexOf(after);
  var stop = index;
  var currentLine = after.loc.end.line;
  for (var i = stop; i < sorted.length; i += 1) {
    var endLine = sorted[i].loc.end.line;
    if (endLine === currentLine) {
      stop = i;
    } else {
      break;
    }
  }
  return sorted.slice(index + 1, stop + 1);
}

function enumerate(node, func, options) {
  if (!node) {
    return node;
  }
  if (node.type) {
    func(node, options || {});
  }
  (0, _entries2.default)(node).forEach(function (_ref) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
        key = _ref2[0],
        val = _ref2[1];

    if (val && val.type) {
      enumerate(val, func, { parent: node, property: key, isArray: false });
    } else if (Array.isArray(val)) {
      var nodes = _underscore2.default.clone(val);
      nodes.forEach(function (n, index) {
        return enumerate(n, func, {
          parent: node, property: key, isArray: true, index: index
        });
      });
    }
  });
  return node;
}

function performOffset(node, offset) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  if (!node || !node.loc) {
    throw new Error('performOffset must be given a node with a loc');
  }
  var loc = node.loc;
  var startingAt = options.startingAt;


  if (offset.line && !options.both) {
    offset.column = 0;
  }

  var diff = _glimmerSyntax.builders.loc(offset.line, offset.column, offset.line, offset.column);

  if (startingAt) {
    // entire node is before start
    if ((0, _location.posBefore)(loc.end, startingAt)) {
      diff = null;
    }

    // on line below start
    if (loc.start.line > startingAt.line) {
      diff.start.column = 0;
      diff.end.column = 0;
    }

    if (loc.end.line > startingAt.line) {
      if (loc.start.line === startingAt.line) {
        // starts on same line after but ends on lower line
        // shift start but not end
        diff.end.column = 0;
      } else {
        // node ends below line than starting line
        diff.end.column = 0;
      }
    }

    // node starts before start but ends after
    if ((0, _location.posBefore)(loc.start, startingAt) && (0, _location.posAfter)(loc.end, startingAt)) {
      diff.start.line = 0;
      diff.start.column = 0;
    }
  }

  if (!diff) {
    return node;
  }

  loc = _glimmerSyntax.builders.loc(loc);
  loc = (0, _location.locAdd)(loc, diff);
  node.loc = loc;
  return node;
}

function offsetNodeRecursive(node, offset, options) {
  if (!node) {
    return node;
  }
  if (node.loc) {
    performOffset(node, offset, options);
  }
  (0, _values2.default)(node).forEach(function (val) {
    if (val && val.type) {
      offsetNodeRecursive(val, offset, options);
    } else if (Array.isArray(val)) {
      val.forEach(function (v) {
        return offsetNodeRecursive(v, offset, options);
      });
    }
  });
  return node;
}

function offsetNode(node, offset, options) {
  if (Array.isArray(node)) {
    return node.map(function (n) {
      return offsetNode(n, offset, options);
    });
  }
  if (!node || !node.loc) {
    throw new Error('offsetNode must be given a node with a loc');
  }
  if (options && options.recursive) {
    return offsetNodeRecursive(node, offset, options);
  }
  return performOffset(node, offset, options);
}

function removeNode(target, ast) {
  enumerate(ast, function (node, _ref3) {
    var parent = _ref3.parent,
        property = _ref3.property,
        isArray = _ref3.isArray,
        index = _ref3.index;

    if (target === node) {
      if (isArray) {
        parent[property].splice(index, 1);
      } else {
        delete parent[property];
      }
    }
  });

  var startingAt = target.loc.start;
  var span = (0, _location.locSpan)(target.loc);
  span.line = -span.line;
  span.column = -span.column;
  var after = nodesAfter(target, ast);
  var before = nodesAfter(target, ast);
  offsetNode(ast, span, { recursive: true, startingAt: startingAt });
  if (after.length === 0 && before.length === 0) {
    // TODO: delete empty line
  }

  return ast;
}

function sizeOfNodes(nodes) {
  var minLine = null;
  var maxLine = null;
  var minCol = null;
  var maxCol = null;
  enumerate({ nodes: nodes }, function (n) {
    if (n.loc) {
      var _n$loc = n.loc,
          _n$loc$start = _n$loc.start,
          startLine = _n$loc$start.line,
          startCol = _n$loc$start.column,
          _n$loc$end = _n$loc.end,
          endLine = _n$loc$end.line,
          endCol = _n$loc$end.column;

      if (minLine === null || minLine > startLine) {
        minLine = startLine;
      }
      if (maxLine === null || maxLine < endLine) {
        maxLine = endLine;
      }
      if (minCol === null || minCol > startCol) {
        minCol = startCol;
      }
      if (maxCol === null || maxCol < endCol) {
        maxCol = endCol;
      }
    }
  }, { recursive: true });
  return _glimmerSyntax.builders.loc(0, 0, maxLine - minLine, maxCol - minCol).end;
}

exports.sortNodes = sortNodes;
exports.nodesBefore = nodesBefore;
exports.nodesAfter = nodesAfter;
exports.nodeIndex = nodeIndex;
exports.offsetNode = offsetNode;
exports.removeNode = removeNode;
exports.enumerate = enumerate;
exports.sizeOfNodes = sizeOfNodes;